import { Any } from 'ts-toolbelt';

declare class SubmoduleController {
}
type ControllerSignal = SubmoduleController | Promise<SubmoduleController>;
declare class StopWithError extends SubmoduleController {
    phase: 'definition' | 'config' | 'services' | 'route' | 'router' | 'start' | 'runtime';
    exitCode: number;
    exitReason: string;
    error: any;
    constructor(phase: 'definition' | 'config' | 'services' | 'route' | 'router' | 'start' | 'runtime', exitCode: number, exitReason: string, error: any);
}
declare const stopAndDisplayHelp: (helpMessage: string, exitCode?: number) => SubmoduleController;
declare const stopWithError: (phase: StopWithError['phase'], exitCode?: StopWithError['exitCode'], exitReason?: StopWithError['exitReason'], error?: any) => SubmoduleController;

interface ExtractInputOutput<T = unknown> {
    routeModule: T;
    input: unknown;
    output: unknown;
}
type Submodule<InitArgs = {}, Config = {}, Services = {}> = Any.Compute<{
    createConfig?: (createConfigParams?: {
        initArgs: InitArgs;
    }) => Config | Promise<Config>;
    createServices?: (createServiceParams: {
        initArgs: InitArgs;
        config: Config;
    }) => Services | Promise<Services>;
}, "flat">;
type ExecutableSubmodule<InitArgs = {}, Config = {}, Services = {}, RouteModule = unknown, Input = unknown, Output = unknown> = Submodule<InitArgs, Config, Services> & Any.Compute<{
    loadRouteModule?: (loadRouteModuleParams: {
        initArgs: InitArgs;
        config: Config;
        query: string;
        preloadedRoutes?: any;
    }) => RouteModule | Promise<RouteModule> | undefined | Promise<undefined>;
    execute: (executeParams: {
        initArgs: InitArgs;
        config: Config;
        services: Services;
        route: RouteModule;
        query?: string;
        input: Input;
    }) => Promise<Output>;
}, 'flat'>;
type RouteLike<Context = unknown, RouteModule = unknown> = {
    handle: (context: Context) => unknown | Promise<unknown>;
    routeModule: RouteModule;
    routeName: string;
};
type ServerSubmodule<InitArgs = {}, Config = {}, Services = {}, Context = unknown, RouteModule = unknown, Route extends RouteLike<Context, RouteModule> = RouteLike<Context, RouteModule>, Router extends Record<string, Route> = Record<string, Route>> = Submodule<InitArgs, Config, Services> & Any.Compute<{
    loadRouteModules?: (loadRouteModuleParams: {
        initArgs: InitArgs;
        config: Config;
    }) => Record<string, RouteModule> | Promise<Record<string, RouteModule>>;
    createRoute: (createRouteParams: {
        initArgs: InitArgs;
        config: Config;
        services: Services;
        routeModule: RouteModule;
        routeName: string;
    }) => Promise<Route | Route[] | undefined> | (Route | Route[] | undefined);
    createRouter?: (createRouterParams: {
        initArgs: InitArgs;
        config: Config;
        services: Services;
        routes: Record<string, Route>;
    }) => Router | Promise<Router>;
    createInnerClient?: (createInnerClientParams: {
        initArgs: InitArgs;
        config: Config;
        services: Services;
        router: Router;
        startPromise: Promise<any>;
    }) => Promise<(query: string, input: unknown) => unknown | Promise<unknown>>;
    serve: (input: {
        initArgs: InitArgs;
        config: Config;
        services: Services;
        router: Router;
    }) => Promise<void>;
}, "flat">;
declare const createExecutable: <Extractor extends ExtractInputOutput<unknown>, Routes = unknown>(submoduleDef: ExecutableSubmodule<any, any, any, any>, initArgs: any) => <Query extends keyof Routes, IO extends Extractor & {
    routeModule: Routes[Query];
} = Extractor & {
    routeModule: Routes[Query];
}>(query: Query | Omit<string, Query>, input: IO["input"]) => Promise<IO["output"]>;
type Executable<InitArgs, Config, Services, Input, Output> = (executableParam: {
    config: Config;
    services: Services;
    input: Input extends undefined ? undefined : Input;
    initArgs: InitArgs;
}) => Output | Promise<Output>;
declare const prepareExecutable: <InitArgs, Config, Services>(submoduleDef: ExecutableSubmodule<InitArgs, Config, Services, Executable<InitArgs, Config, Services, any, any>, unknown, unknown>, initArgs: InitArgs) => <Input, Output>(executable: Executable<InitArgs, Config, Services, Input, Output>, input?: Input | undefined) => Promise<Output>;
interface ExtractRouteFn<RouteModule = unknown> {
    routeModule: RouteModule;
    routeFn: unknown;
}
declare const serve: (nonValidatedSubmodule: ServerSubmodule<any, any, any, any, any, any, any>, initArgs: any) => Promise<{
    submodule: {
        createConfig: (createConfigParams?: {
            initArgs: any;
        } | undefined) => any;
        createServices: (createServiceParams: {
            initArgs: any;
            config: any;
        }) => any;
        loadRouteModules: (loadRouteModuleParams: {
            initArgs: any;
            config: any;
        }) => Record<string, any> | Promise<Record<string, any>>;
        createRoute: (createRouteParams: {
            initArgs: any;
            config: any;
            services: any;
            routeModule: any;
            routeName: string;
        }) => any;
        createRouter: (createRouterParams: {
            initArgs: any;
            config: any;
            services: any;
            routes: Record<string, any>;
        }) => any;
        createInnerClient: (createInnerClientParams: {
            initArgs: any;
            config: any;
            services: any;
            router: any;
            startPromise: Promise<any>;
        }) => Promise<(query: string, input: unknown) => unknown | Promise<unknown>>;
        serve: (input: {
            initArgs: any;
            config: any;
            services: any;
            router: any;
        }) => Promise<void>;
        loadRouteModule(): Promise<undefined>;
        execute(): Promise<never>;
    };
    config: any;
    services: any;
    router: any;
    startPromise: Promise<void>;
    client: (query: string, input: unknown) => unknown | Promise<unknown>;
}>;
type DefaultRouteModule<CallContext, Output = unknown> = {
    default: (callContext: CallContext) => Output | Promise<Output>;
};
interface DefaultRouteFnExtractor<CallContext> extends ExtractRouteFn<DefaultRouteModule<CallContext>> {
    routeFn: this["routeModule"]["default"];
}
interface DefaultInputOutputExtractor<RouteModule extends DefaultRouteModule<any>> extends ExtractInputOutput<RouteModule> {
    input: Parameters<this["routeModule"]["default"]>[0] extends {
        input: infer Input;
    } ? Input : unknown;
    output: ReturnType<this["routeModule"]["default"]> extends Promise<infer Result> ? Result : ReturnType<this["routeModule"]["default"]>;
}
declare class SubmoduleBuilder<InitArgs = {}, Config = {}, Services = {}, Context = {}, RouteModule = unknown, Route extends RouteLike<Context, RouteModule> = RouteLike<Context, RouteModule>, RouteFn = unknown, Routes = unknown, Extractor extends ExtractInputOutput = ExtractInputOutput> {
    routes<Routes>(): SubmoduleBuilder<InitArgs, Config, Services, Context, RouteModule, Route, RouteFn, Routes, Extractor>;
    init<InitType>(): SubmoduleBuilder<InitType, Config, Services, Context, RouteModule, Route, RouteFn, Routes, Extractor>;
    config<ConfigType>(): SubmoduleBuilder<InitArgs, ConfigType, Services, Context, RouteModule, Route, RouteFn, Routes, Extractor>;
    services<ServicesType>(): SubmoduleBuilder<InitArgs, Config, ServicesType, Context, RouteModule, Route, RouteFn, Routes, Extractor>;
    context<ContextType>(): SubmoduleBuilder<InitArgs, Config, Services, ContextType, RouteModule, RouteLike<ContextType, RouteModule>, RouteFn, Routes, Extractor>;
    routeModule<RouteModuleType, RouteFnExtractor extends ExtractRouteFn>(): SubmoduleBuilder<InitArgs, Config, Services, Context, RouteModuleType, RouteLike<Context, RouteModuleType>, (RouteFnExtractor & {
        routeModule: RouteModuleType;
    })['routeFn'], Routes, Extractor>;
    route<RouteType extends RouteLike<Context, RouteModule>>(): SubmoduleBuilder<InitArgs, Config, Services, Context, RouteModule, RouteType, RouteFn, Routes, Extractor>;
    extractor<ExtractorType extends ExtractInputOutput>(): SubmoduleBuilder<InitArgs, Config, Services, Context, RouteModule, Route, RouteFn, Routes, ExtractorType>;
    serverSubmodule: ServerSubmodule<InitArgs, Config, Services, Context, RouteModule, Route, Record<string, Route>>;
    executableSubmodule: ExecutableSubmodule<InitArgs, Config, Services, RouteModule>;
    types: {
        initArgs: InitArgs;
        config: Config;
        services: Services;
        context: Context;
        route: Route;
        routeFn: RouteFn;
        routeModule: RouteModule;
        queries: keyof Routes;
    };
    defineRouteFn<RouteFnType extends RouteFn>(routeFn: RouteFnType): RouteFnType;
    defineExecutable(submoduleDef: ExecutableSubmodule<InitArgs, Config, Services, RouteModule>): ExecutableSubmodule<InitArgs, Config, Services, RouteModule, unknown, unknown>;
    createExecutable(submoduleDef: ExecutableSubmodule<InitArgs, Config, Services, RouteModule>, initArgs?: InitArgs): <Query extends keyof Routes, IO extends Extractor & {
        routeModule: Routes[Query];
    } = Extractor & {
        routeModule: Routes[Query];
    }>(query: Query | Omit<string, Query>, input: IO["input"]) => Promise<IO["output"]>;
    prepareExecutable(submoduleDef: ExecutableSubmodule<InitArgs, Config, Services, Executable<InitArgs, Config, Services, any, any>>, initArgs: InitArgs): <Input, Output>(executable: Executable<InitArgs, Config, Services, Input, Output>, input?: Input | undefined) => Promise<Output>;
    defineServer(submoduleDef: ServerSubmodule<InitArgs, Config, Services, Context, RouteModule, Route, Record<string, Route>>): ServerSubmodule<InitArgs, Config, Services, Context, RouteModule, Route, Record<string, Route>>;
    serve(submoduleDef: ServerSubmodule<InitArgs, Config, Services, Context, RouteModule, Route, Record<string, Route>>, initArgs?: InitArgs): Promise<{
        submodule: {
            createConfig: (createConfigParams?: {
                initArgs: any;
            } | undefined) => any;
            createServices: (createServiceParams: {
                initArgs: any;
                config: any;
            }) => any;
            loadRouteModules: (loadRouteModuleParams: {
                initArgs: any;
                config: any;
            }) => Record<string, any> | Promise<Record<string, any>>;
            createRoute: (createRouteParams: {
                initArgs: any;
                config: any;
                services: any;
                routeModule: any;
                routeName: string;
            }) => any;
            createRouter: (createRouterParams: {
                initArgs: any;
                config: any;
                services: any;
                routes: Record<string, any>;
            }) => any;
            createInnerClient: (createInnerClientParams: {
                initArgs: any;
                config: any;
                services: any;
                router: any;
                startPromise: Promise<any>;
            }) => Promise<(query: string, input: unknown) => unknown>;
            serve: (input: {
                initArgs: any;
                config: any;
                services: any;
                router: any;
            }) => Promise<void>;
            loadRouteModule(): Promise<undefined>;
            execute(): Promise<never>;
        };
        config: any;
        services: any;
        router: any;
        startPromise: Promise<void>;
        client: (query: string, input: unknown) => unknown;
    }>;
}
declare const builder: () => SubmoduleBuilder<{}, {}, {}, {}, unknown, RouteLike<{}, unknown>, unknown, unknown, ExtractInputOutput<unknown>>;

export { ControllerSignal, DefaultInputOutputExtractor, DefaultRouteFnExtractor, DefaultRouteModule, ExecutableSubmodule, ExtractInputOutput, ExtractRouteFn, RouteLike, ServerSubmodule, Submodule, SubmoduleController, builder, createExecutable, prepareExecutable, serve, stopAndDisplayHelp, stopWithError };
